/********************************************************************************************************/
/*
 Copyright (C) 2008 Biger Kevin, Marguet Quentin, Muller Thomas, Vinatier Frdric (University of Science and Technology of Nantes)

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along
 with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

 Contact: glovecontrol@free.fr
*/
/********************************************************************************************************/
//
#include "Events.h"
//
#ifdef WIN32 // l'ordinateur est sous windows
//
namespace AAC {
namespace EE {
//
/*************************************************************************/
//
Events::~Events(){
}
//
/*************************************************************************/
//
int Events::clickGaucheDown(const CvPoint& pt){     // Appel systme d'un clic gauche
    DWORD dwEventFlags;
	dwEventFlags = MOUSEEVENTF_ABSOLUTE;
	dwEventFlags |= MOUSEEVENTF_LEFTDOWN;
	mouse_event(dwEventFlags,pt.x,pt.y,0,0);
}
//
/*************************************************************************/
//
int Events::clickGaucheUp(const CvPoint& pt){     // Appel systme d'un clic gauche
    DWORD dwEventFlags;
	dwEventFlags = MOUSEEVENTF_ABSOLUTE;
	dwEventFlags |= MOUSEEVENTF_LEFTUP;
	mouse_event(dwEventFlags,pt.x,pt.y,0,0);
}
//
/*************************************************************************/
//
int Events::clickDroitDown(const CvPoint& pt){     // Appel systme d'un clic droit
    DWORD dwEventFlags;
	dwEventFlags = MOUSEEVENTF_ABSOLUTE;
	dwEventFlags |= MOUSEEVENTF_RIGHTDOWN;
	mouse_event(dwEventFlags,pt.x,pt.y,0,0);
}
//
/*************************************************************************/
//
int Events::clickDroitUp(const CvPoint& pt){     // Appel systme d'un clic droit
    DWORD dwEventFlags;
    dwEventFlags = MOUSEEVENTF_ABSOLUTE;
    dwEventFlags |= MOUSEEVENTF_RIGHTUP;
    mouse_event(dwEventFlags,pt.x,pt.y,0,0);
}
//
/*************************************************************************/
//
int Events::doubleClick(const CvPoint& pt){
    Events::clickGaucheDown(pt) ;
    Events::clickGaucheUp(pt) ;
    Events::clickGaucheDown(pt) ;
}
//
/*************************************************************************/
//
int Events::deplaceCurseurPt(const CvPoint& pt){
    SetCursorPos(pt.x,pt.y);
}
//
/*************************************************************************/
//
int Events::glisseCurseurGauchePt(const CvPoint& pt){
    SetCursorPos(pt.x,pt.y);
}
//
/*************************************************************************/
//
int Events::glisseCurseurDroitPt(const CvPoint& pt){
    SetCursorPos(pt.x,pt.y);
}
//
/*************************************************************************/
//
} // EE
} // AAC
//
/********************************************************************************************************/
//
#else // Ordinateur sous autre chose que windows
//
namespace AAC {
namespace EE {
//
/*************************************************************************/
//
CGPoint cvtPoint(CvPoint pt){
	CGPoint point;
	point.x = pt.x;
	point.y = pt.y;
	return point;
}
//
/*************************************************************************/
//
Events::~Events(){
}
//
/*************************************************************************/
//
int Events::clickGaucheDown(const CvPoint& pt){
	DeplaceCurseurPt(pt);
	CGEventRef event;
	event = CGEventCreateMouseEvent (
			NULL, //laisser  null
			kCGEventLeftMouseDown, //bouton gauche enfonce
			cvtPoint(pt),
			kCGMouseButtonLeft
	);  //event cree
	CGEventPost(
			kCGHIDEventTap, //tap general
			event //event a poster
	); //event poste
	CFRelease(event); //on libere l'event de la memoire
	return 0;
}
//
/*************************************************************************/
//
int Events::clickGaucheUp(const CvPoint& pt){
	DeplaceCurseurPt(pt);
	CGEventRef event;
	event = CGEventCreateMouseEvent (
			NULL, //laisser  null
			kCGEventLeftMouseUp, //bouton gauche releve
			cvtPoint(pt),
			kCGMouseButtonLeft
	);  //event cree
	CGEventPost(
			kCGHIDEventTap, //tap general
			event //event a poster
	); //event poste
	CFRelease(event); //on libere l'event de la memoire
	return 0;
}
//
/*************************************************************************/
//
int Events::clickDroitDown(const CvPoint& pt){
	DeplaceCurseurPt(pt);
	CGEventRef event;
	event = CGEventCreateMouseEvent (
			NULL, //laisser  null
			kCGEventRightMouseDown, //bouton droit enfonce
			cvtPoint(pt),
			kCGMouseButtonRight
	);  //event cree
	CGEventPost(
			kCGHIDEventTap, //tap general
			event //event a poster
	); //event poste
	CFRelease(event); //on libere l'event de la memoire
	return 0;
}
//
/*************************************************************************/
//
int Events::clickDroitUp(const CvPoint& pt){
	DeplaceCurseurPt(pt);
	CGEventRef event;
	event = CGEventCreateMouseEvent (
			NULL, //laisser  null
			kCGEventRightMouseUp, //bouton droit enfonce
			cvtPoint(pt),
			kCGMouseButtonRight
	);  //event cree
	CGEventPost(
			kCGHIDEventTap, //tap general
			event //event a poster
	); //event poste
	CFRelease(event); //on libere l'event de la memoire
	return 0;
}
//
/*************************************************************************/
//
int Events::doubleClick(const CvPoint& pt){
	 DeplaceCurseurPt(pt);
 	CGPostMouseEvent( cvtPoint(pt), false, 1, true );
 	CGPostMouseEvent( cvtPoint(pt), false, 1, false );
 	CGPostMouseEvent( cvtPoint(pt), false, 1, true );
 	CGPostMouseEvent( cvtPoint(pt), false, 1, false );

	return 0;
}
//
/*************************************************************************/
//
int Events::deplaceCurseurPt(const CvPoint& pt){
	CGEventRef event;
	event = CGEventCreateMouseEvent (
		NULL, //laisser  null
		kCGEventMouseMoved, //event deplacement souris
		cvtPoint(pt),
		kCGMouseButtonRight //pas de mask specifique au deplacement, a ameliorer peut-etre
	);
	CGEventPost(
		kCGHIDEventTap, //tap general
		event //event a poster
	); //event poste
	CFRelease(event); //on libere l'event de la memoire
	return 0;
}
//
/*************************************************************************/
//
int Events::glisseCurseurGauchePt(const CvPoint& pt){

	CGEventRef event;
	event = CGEventCreateMouseEvent (
									 NULL, //laisser  null
									 kCGEventLeftMouseDragged, //bouton gauche glisse
									 cvtPoint(pt),
									 kCGMouseButtonLeft
									 );  //event cree
	CGEventPost(
				kCGHIDEventTap, //tap general
				event //event a poster
				); //event poste
	CFRelease(event); //on libere l'event de la memoire
	return 0;

}
//
/*************************************************************************/
//
int Events::glisseCurseurDroitPt(const CvPoint& pt){
	CGEventRef event;
	event = CGEventCreateMouseEvent (
									 NULL, //laisser  null
									 kCGEventRightMouseDragged, //bouton droit glisse
									 cvtPoint(pt),
									 kCGMouseButtonRight
									 );  //event cree
	CGEventPost(
				kCGHIDEventTap, //tap general
				event //event a poster
				); //event poste
	CFRelease(event); //on libere l'event de la memoire
	return 0;
}
//
/*************************************************************************/
//
} // EE
} // AAC
//
/********************************************************************************************************/
//
#endif
//
/********************************************************************************************************/

/*
//
// Ancien code Mac OS X
//
int Events::doubleclick(CvPoint pt){
		DeplaceCurseurPt(pt);
		CGEventRef event;
		event = CGEventCreateMouseEvent (
										 NULL, //laisser  null
										 kCGEventLeftMouseDown, //bouton droit enfonce
										 cvtPoint(pt),
										 kCGMouseButtonLeft
										 );  //event cree
		DeplaceCurseurPt(pt);
		CGEventPost(
					kCGHIDEventTap, //tap general
					event //event a poster
					); //event poste
		DeplaceCurseurPt(pt);
		event = CGEventCreateMouseEvent (
										 NULL, //laisser  null
										 kCGEventLeftMouseUp, //bouton droit enfonce
										 cvtPoint(pt),
										 kCGMouseButtonLeft
										 );  //event cree
		DeplaceCurseurPt(pt);
		CGEventPost(
					kCGHIDEventTap, //tap general
					event //event a poster
					); //event poste
		DeplaceCurseurPt(pt);
		event = CGEventCreateMouseEvent (
										 NULL, //laisser  null
										 kCGEventLeftMouseDown, //bouton droit enfonce
										 cvtPoint(pt),
										 kCGMouseButtonLeft
										 );  //event cree
		DeplaceCurseurPt(pt);
		CGEventPost(
					kCGHIDEventTap, //tap general
					event //event a poster
					); //event poste
		DeplaceCurseurPt(pt);
		event = CGEventCreateMouseEvent (
										 NULL, //laisser  null
										 kCGEventLeftMouseUp, //bouton droit enfonce
										 cvtPoint(pt),
										 kCGMouseButtonLeft
										 );  //event cree
		DeplaceCurseurPt(pt);
		CGEventPost(
					kCGHIDEventTap, //tap general
					event //event a poster
					); //event poste
		DeplaceCurseurPt(pt);
		CFRelease(event); //on libere l'event de la memoire
		return 0;
}

	int Events::doubleClick(CvPoint pt){
		DeplaceCurseurPt(pt);
			CGPostKeyboardEvent( (CGCharCode)0, (CGKeyCode)55, true );    //  command down
			CGPostKeyboardEvent( (CGCharCode)0, (CGKeyCode)31, true );      //  key down
			CGPostKeyboardEvent( (CGCharCode)0, (CGKeyCode)31, false );    //  key up
			CGPostKeyboardEvent( (CGCharCode)0, (CGKeyCode)55, false );    //  command up
		return 0;
	}
*/

/*
//
// Ancien code Windows
//
int Events::deplaceCurseurPtAPt(const CvPoint& ptdep,const CvPoint& ptdest){
    if(false){
        bresenham(ptdep,ptdest );
    } else { SetCursorPos(ptdest.x,ptdest.y);}
}
int Events::bresenham(const CvPoint& currentposition, const CvPoint& finalposition ){
    int y=currentposition.y;
    int deltax=0,deltay=0;
    deltax=finalposition.x-currentposition.x;
    deltay=finalposition.y-currentposition.y;
    int inc=2*deltay-deltax;
    for(int x=currentposition.x;x<finalposition.x;x++){
        Sleep(1);
        SetCursorPos(x,y);
        inc=inc+2*deltay;
        if(inc>0){
            y++;
            inc=inc+2*(deltay-deltax);}
            else {inc=inc+2*deltay;}
        }
     return 0;
}
*/
