/********************************************************************************************************/
/*
Copyright (C) 2008 Biger Kevin, Marguet Quentin, Muller Thomas, Vinatier Frédéric (University of Science and Technology of Nantes)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

Contact: glovecontrol@free.fr
*/
/********************************************************************************************************/
//
#include "Tray.h"
//
using namespace AAC;
//
namespace GC{
//
/*************************************************************************/
//
Tray::Tray(){
    if (!QSystemTrayIcon::isSystemTrayAvailable()) {
        QMessageBox::critical(0, QString::fromUtf8("Erreur"),
                              QString::fromUtf8("Impossible de détecter la zone de notification"));
        exit(0);
    }
    initVariables();
    createActions();
    createMenuIcon();
    running=false;

}
//
/*************************************************************************/
//
Tray::~Tray(){
}
//
/*************************************************************************/
//
void Tray::start(){
    show();
    terminateCalibrate();
}
//
/*************************************************************************/
//
void Tray::initVariables(){
    calibrateWindow=0;
    acquisition=0;
}
//
/*************************************************************************/
//
void Tray::createActions(){

    runstopAction = new QAction(tr("<^-_-^>"), this);
    runstopAction->setIcon(QIcon(":/images/icone_bleue.png"));
    connect(runstopAction, SIGNAL(triggered()), this, SLOT(startstop()));

    calibrateAction = new QAction(QString::fromUtf8("&Calibrage"), this);
    calibrateAction->setIcon(QIcon(":/images/icone_mauve.png"));
    connect(calibrateAction, SIGNAL(triggered()), this, SLOT(calibrate()));

    helpAction = new QAction(QString::fromUtf8("&Aide"), this);
    helpAction->setIcon(QIcon(":/images/icone_jaune.png"));
    connect(helpAction, SIGNAL(triggered()), this, SLOT(help()));

    quitAction = new QAction(QString::fromUtf8("&Quitter"), this);
    quitAction->setIcon(QIcon(":/images/icone_rouge.png"));
    connect(quitAction, SIGNAL(triggered()), qApp, SLOT(quit()));
}
//
/*************************************************************************/
//
void Tray::createMenuIcon(){
    trayMenu = new QMenu();

    trayMenu->addAction(runstopAction);
    trayMenu->addSeparator();
    trayMenu->addAction(calibrateAction);
    trayMenu->addSeparator();
    trayMenu->addAction(helpAction);
    trayMenu->addSeparator();
    trayMenu->addAction(quitAction);



    this->setContextMenu(trayMenu);
    this->setToolTip(QString::fromUtf8("GloveControl"));

    connect(this, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));
}
//
/*************************************************************************/
//
void Tray::startstop(){
    if(running){
        runstopAction->setText(QString::fromUtf8("&Start"));
        calibrateAction->setEnabled(true);
        this->setIcon(QIcon(":/images/icone_bleue.png"));
        runstopAction->setIcon(QIcon(":/images/icone_bleue.png"));
        running=false;
        acquisition->setState(Zor::Tools::ARRET);
        while(!acquisition->isRunning()){}
    } else {
        runstopAction->setText(QString::fromUtf8("&Stop"));
        calibrateAction->setEnabled(false);
        this->setIcon(QIcon(":/images/icone_verte.png"));
        runstopAction->setIcon(QIcon(":/images/icone_verte.png"));
        running=true;
        if(acquisition) delete(acquisition);
        acquisition = new Acquisition();
        acquisition->start();
    }
}
//
/*************************************************************************/
//
void Tray::calibrate(){
    calibrateAction->setEnabled(false);
    runstopAction->setEnabled(false);
    quitAction->setEnabled(false);
    if(calibrateWindow) delete(calibrateWindow);
    calibrateWindow = new UIZor();
    connect(calibrateWindow, SIGNAL(terminate()), this, SLOT(terminateCalibrate()));
    calibrateWindow->show();
}
//
/*************************************************************************/
//
void Tray::help(){
}
//
/*************************************************************************/
//
void Tray::iconActivated(QSystemTrayIcon::ActivationReason reason){
    switch (reason){
        case QSystemTrayIcon::Unknown:
        break;
        case QSystemTrayIcon::Trigger:
        break;
        case QSystemTrayIcon::DoubleClick:
        break;
        case QSystemTrayIcon::MiddleClick:
        break;
        default:
        break;
    }
}
//
/*************************************************************************/
//
void Tray::terminateCalibrate(){
    calibrateWindow->disconnect();
    calibrateAction->setEnabled(true);

    bool calibrageCouleurs = QFile::exists(QDir::currentPath()+"/calibrage/calibrageCouleurs.gc");
    bool calibrageMouvement = QFile::exists(QDir::currentPath()+"/calibrage/calibrageMouvement.gc");
    if(calibrageCouleurs && calibrageMouvement){
        runstopAction->setText(QString::fromUtf8("&Start"));
        setIcon(QIcon(":/images/icone_bleue.png"));
        runstopAction->setEnabled(true);
		showMessage(
            QString::fromUtf8("Glove Control"),
            QString::fromUtf8("Vous pouvez maintenant utiliser Glove Control"),
            QSystemTrayIcon::NoIcon,
            5000
        );
    } else if(calibrageCouleurs){
        runstopAction->setText("<^-_-^>");
        runstopAction->setEnabled(false);
        setIcon(QIcon(":/images/icone_jaune.png"));
        showMessage(
            QString::fromUtf8("Glove Control"),
            QString::fromUtf8("Attention, pour pouvoir utiliser Glove Control vous devez calibrer le mouvement."),
            QSystemTrayIcon::Warning,
            5000
        );
    } else if(calibrageMouvement){
        runstopAction->setText("<^-_-^>");
        runstopAction->setEnabled(false);
        setIcon(QIcon(":/images/icone_jaune.png"));
        showMessage(
            QString::fromUtf8("Glove Control"),
            QString::fromUtf8("Attention, pour pouvoir utiliser Glove Control vous devez calibrer les couleurs."),
            QSystemTrayIcon::Warning,
            5000
        );
    } else {
        runstopAction->setText("<^-_-^>");
        runstopAction->setEnabled(false);
        setIcon(QIcon(":/images/icone_rouge.png"));
        showMessage(
            QString::fromUtf8("Glove Control"),
            //QString::fromUtf8("Bienvenue, pour pouvoir utiliser Glove Control vous devez dans un premier temps effectuer le calibrage du logiciel."),
            QString::fromUtf8("Pour pouvoir utiliser Glove Control vous devez procéder au calibrage."),
            QSystemTrayIcon::Information,
            5000
        );
    }
    quitAction->setEnabled(true);
}
//
/*************************************************************************/
//
}   // GC
//
/*************************************************************************/
//

