/********************************************************************************************************/
/*
Copyright (C) 2008 Biger Kevin, Marguet Quentin, Muller Thomas, Vinatier Frédéric (University of Science and Technology of Nantes)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

Contact: glovecontrol@free.fr
*/
/********************************************************************************************************/
//
#include "UIZor.h"
//
using namespace Zor;
using namespace Zor::Tools;
//
/********************************************************************************************************/
// Constructeur de la classe
UIZor::UIZor( QWidget* parent ): QWidget( parent ) {
	// Creation des differents elements composant l'interface (QTDesigner)
	setupUi( this );
    // Connection des signaux et slots
    connect(this, SIGNAL(updateText(const QString&)), textBrowser_Control, SLOT(setPlainText(const QString &)));
    // Creation de la fenêtre ou sera affiché l'image pour le premier calibrage
	_DisplayCouleurs = new DisplayCouleurs(groupBox_Visualisation);
    _DisplayCouleurs->setGeometry(QRect(15, 25, 640, 480));
    _DisplayCouleurs->setObjectName(QString::fromUtf8("Fenetre de visualisation"));
    _DisplayCouleurs->setEnabled(false);
    // Creation de la fenêtre ou sera affiché l'image pour le second calibrage
	_DisplayMouvement = new DisplayMouvement(groupBox_Visualisation);
    _DisplayMouvement->setGeometry(QRect(15, 25, 640, 480));
    _DisplayMouvement->setObjectName(QString::fromUtf8("Fenetre de visualisation"));
    _DisplayMouvement->setEnabled(false);
    // Creation du peripherique d'acquisition de la webcam
    _Webcam = new Webcam();
    connect(this->_Webcam, SIGNAL(updateFPS(const QString&)),this->lineEdit_FPS, SLOT(setText(const QString&)));
    // Creation de la classe qui calcule le seuil
    _Seuil = new Seuil(_DisplayCouleurs);
    // Lancement des différentes étapes de calibration
    start();

}
//
/********************************************************************************************************/
// Destructeur de la classe
UIZor::~UIZor() {
    if(_DisplayMouvement) delete(_DisplayMouvement);
    if(_DisplayCouleurs) delete(_DisplayCouleurs);
    if(_Webcam) delete(_Webcam);
    if(_Seuil) delete(_Seuil);
}
//
/********************************************************************************************************/
// Redefinition de l'étape de fermeture de la fenêtre
void UIZor::closeEvent(QCloseEvent *event){
    event->ignore();
    if(_Webcam) _Webcam->setEtatWebcam(Tools::ARRET);
    while(_Webcam->isRunning()){}
    setVisible(false);
    emit terminate();
}
//
/********************************************************************************************************/
//
void UIZor::start(){
    // Connexion/Deconnexion des signaux et slots
    pushButton_Control->disconnect();
    connect(pushButton_Control, SIGNAL(clicked()), this, SLOT(etapeChoix()) );
    // Mise à jour des informations sur les boutons et le menu déroulant
    comboBox_Heuristique->setEnabled(false);
    comboBox_Heuristique->addItem(QString::fromUtf8("Détection large"),HEU_0);
    comboBox_Heuristique->addItem(QString::fromUtf8("Détection fine"),HEU_1);
    comboBox_Etape->setEnabled(true);
    comboBox_Etape->addItem(QString::fromUtf8("Calibrage des couleurs"),0);
    comboBox_Etape->addItem(QString::fromUtf8("Calibrage du mouvement"),1);
    pushButton_Rotation->setEnabled(false);
    pushButton_Rotation->setText(QString::fromUtf8("Rotation"));
    pushButton_Control->setEnabled(true);
    pushButton_Control->setText(QString::fromUtf8("Commencer"));
    // Mise à jour du texte d'aide

    emit updateText(QString::fromUtf8(
        "Bonjour, nous allons procéder au calibrage de votre webcam.\n"
        "Veuillez prendre le temps de bien lire les instructions.\n"
        "Le calibrage se fait en 2 étapes :\n"
        "- calibrage des couleurs ;\n"
        "- calibrage du mouvement.\n"
        "Vous pouvez choisir en bas à gauche, le calibrage a effectuer. Cependant, la première fois, vous devez faire les 2 étapes.\n"
        "Cliquer sur le bouton \"Commencer\" après avoir choisi le calibrage a effectuer.\n"
    ));
}
//
/********************************************************************************************************/
//
void UIZor::etapeChoix(){
    switch(comboBox_Etape->itemData(comboBox_Etape->currentIndex()).toInt()){
        case 0 :
            etape11();
        break;
        case 1 :
            etape21();
        break;
        default :
        break;
    }

}
//
/********************************************************************************************************/
//
void UIZor::etape11(){
    // Desactivation
    comboBox_Etape->setEnabled(false);
    _DisplayMouvement->setVisible(false);
    _DisplayCouleurs->setVisible(true);
    // Chargement du calibrage couleur dans le peripherique
    _Webcam->setDisplay(_DisplayCouleurs);
    // Lancement de la webcam
    _Webcam->changeState();
    // Connexion/Deconnexion des signaux et slots
    pushButton_Control->disconnect();
    connect(pushButton_Control, SIGNAL(clicked()), this->_Webcam, SLOT(changeState()) );
    connect(pushButton_Control, SIGNAL(clicked()), this, SLOT(etape12()) );
    connect(pushButton_Rotation, SIGNAL(clicked()), this->_DisplayCouleurs, SLOT(switchRotation()) );
    // Mise à jour des informations pour les boutons
    pushButton_Rotation->setEnabled(true);
    pushButton_Control->setText(QString::fromUtf8("Capture"));
    // Mise à jour du texte d'aide
    emit updateText(QString::fromUtf8(
        "Vous devriez voir apparaître les images de votre webcam à l'écran. Si ce n'est pas le cas, consultez l'aide.\n"
        "Si l'image est inversé, il vous suffit de retourner l'écran ou d'appuyer sur le bouton \"rotation\".\n"
        "Vous devez tout d'abord positionner la webcam de manière à ce qu'elle soit face à vous.\n"
        "Ensuite, mettez le gant de contrôle puis allumer les diodes.\n"
        "Positionner le gant et vos doigts de tel manière que la webcam affiche les 3 diodes distinctement sur l'image.\n"
        "Ne bouger plus le gant et avec votre autre main, appuyer sur le bouton \"Capture\"."
    ));
}
//
/********************************************************************************************************/
//
void UIZor::etape12(){
    // Connexion/Deconnexion des signaux et slots
    pushButton_Control->disconnect();
    connect(_DisplayCouleurs, SIGNAL(existe(bool)), this, SLOT(existe(bool)) );
    connect(pushButton_Control, SIGNAL(clicked()), this, SLOT(etape13()) );
    // Mise à jour des informations pour les boutons et le menu déroulant
    pushButton_Rotation->setEnabled(false);
    pushButton_Control->setEnabled(false);
    _DisplayCouleurs->setEnabled(true);
    comboBox_Heuristique->setEnabled(true);
    pushButton_Control->setText(QString::fromUtf8("Calcul"));
    // Mise à jour du texte d'aide
    emit updateText(QString::fromUtf8(
        "Vous devez entourer chacune des 3 diodes sur l'image par un rectangle.\n"
        "Pour cela, cliquez gauche, maintenez le clique et entourez la zone qui correspond à la diode.\n"
        "Relacher le bouton de la souris pour valider. Pour annuler un rectangle, il suffit de cliquer droit dessus.\n"
        "Attention : il est important de bien faire cette étape et d'entourer au mieux les 3 diodes.\n"
        "Une fois cela effectué, vous pouvez choisir le type de calibrage dans le menu déroulant.\n"
        "Toutefois, il est conseillé de laisser par défaut et donc de prendre la détection large."
    ));
}
//
/********************************************************************************************************/
//
void UIZor::existe(bool existe){
    pushButton_Control->setEnabled(existe);
}
//
/********************************************************************************************************/
//
void UIZor::etape13(){
    // Connexion/Deconnexion des signaux et slots
    _DisplayCouleurs->disconnect();
    _DisplayCouleurs->setEnabled(false);
    connect(_Seuil, SIGNAL(calculTermine()), this, SLOT(etape14()));
    connect(_Seuil, SIGNAL(rafraichitProgression(int)), progressBar_Seuil, SLOT(setValue(int )));
    // Mise à jour de l'état des boutons
    pushButton_Control->setEnabled(false);
    comboBox_Heuristique->setEnabled(false);
    // Mise à jour du texte d'aide
    emit updateText(QString::fromUtf8(
        "Calcul du seuil en cours, veuillez patienter..."
    ));
    // Calcul du seuil
    progressBar_Seuil->setValue(0);
    _Seuil->setHeuristique((TYPE_HEURISTIQUE)(comboBox_Heuristique->itemData(comboBox_Heuristique->currentIndex()).toInt()));
    _Seuil->start();
}
//
/********************************************************************************************************/
//
void UIZor::etape14(){
    // Connexion/Deconnexion des signaux et slots
    progressBar_Seuil->setValue(0);
    pushButton_Control->disconnect();
    connect(pushButton_Control, SIGNAL(clicked()), this, SLOT(etape15()) );
    // Mise à jour de l'état des boutons
    pushButton_Control->setEnabled(true);
    pushButton_Control->setText(QString::fromUtf8("Enregistrer"));
    _DisplayCouleurs->updateQImage(&(_Seuil->getImageSeuil()));
    _DisplayCouleurs->setEnabled(true);
    // Mise à jour du texte d'aide
    emit updateText(QString::fromUtf8(
        "A l'écran, apparaît une image en noir et blanc qui indique l'emplacement des diodes.\n"
        "Cliquez droit sur les zones que vous avez précèdemment créées pour les faire disparaître.\n"
        "Il doit y avoir des pixels blancs sous chacune de ces zones.\n"
        "Si ce n'est pas le cas, il est préférable de recommencer la phase de calibrage.\n"
        "Cliquer sur le bouton \"Enregistrer\" pour enregistrer le calibrage.\n"
        "Pour procéder au recalibrage, quitter et relancer cette fenêtre."
    ));
}
//
/********************************************************************************************************/
//
void UIZor::etape15(){
    // Connexion/Deconnexion des signaux et slots
    comboBox_Etape->setEnabled(true);
    pushButton_Control->disconnect();
    connect(pushButton_Control, SIGNAL(clicked()), this, SLOT(etapeChoix()) );
    // Mise à jour de l'état des boutons
    pushButton_Control->setEnabled(true);
    pushButton_Control->setText("Retour");
    if(sauvegarde1()){
        emit updateText(QString::fromUtf8(
            "Félicitation, le calibrage des couleurs a été effectué avec succès.\n"
            "Vous pouvez procéder au calibrage du mouvement ou quitter.\n"
        ));
    } else {
        emit updateText(QString::fromUtf8(
            "Le calibrage n'a pas pu être enregistré, consultez l'aide pour plus d'informations.\n"
        ));
    }
    _DisplayCouleurs->setEnabled(false);
    _DisplayCouleurs->effaceElements();
}
//
/********************************************************************************************************/
//
void UIZor::etape21(){
    // Desactivation
    comboBox_Etape->setEnabled(false);
    _DisplayCouleurs->setVisible(false);
    _DisplayMouvement->setVisible(true);
    // Chargement du calibrage couleur dans le peripherique
    _Webcam->setDisplay(_DisplayMouvement);
    // Lancement de la webcam
    _Webcam->changeState();
    // Connexion/Deconnexion des signaux et slots
    pushButton_Control->disconnect();
    connect(pushButton_Control, SIGNAL(clicked()), this->_Webcam, SLOT(changeState()) );
    connect(pushButton_Control, SIGNAL(clicked()), this, SLOT(etape22()) );
    connect(pushButton_Rotation, SIGNAL(clicked()), this->_DisplayMouvement, SLOT(switchRotation()) );
    // Mise à jour des informations pour les boutons
    pushButton_Rotation->setEnabled(true);
    pushButton_Control->setText(QString::fromUtf8("Capture"));
    // Mise à jour du texte d'aide
    emit updateText(QString::fromUtf8(
        "Vous devriez voir apparaître les images de votre webcam à l'écran. Si ce n'est pas le cas, consultez l'aide.\n"
        "Si l'image est inversée, il vous suffit de retourner l'écran ou d'appuyer sur le bouton \"rotation\".\n"
        "Vous devez tout d'abord positionner la webcam de manière à ce qu'elle soit face à vous.\n"
        "Ensuite, mettez le gant de contrôle puis allumer les diodes.\n"
        "Positionner le gant et vos doigts de tel manière que la webcam affiche les 3 diodes distinctement sur l'image.\n"
        "Positionner vos doigts de manière à ce que cette position corresponde à un déplacement (doigts légèrement écartés).\n"
        "Ne bouger plus le gant et avec votre autre main, appuyer sur le bouton \"Capture\"."
    ));
}
//
/********************************************************************************************************/
//
void UIZor::etape22(){
    // Connexion/Deconnexion des signaux et slots
    pushButton_Control->disconnect();
    connect(_DisplayMouvement, SIGNAL(existe(bool)), this, SLOT(existe(bool)) );
    connect(pushButton_Control, SIGNAL(clicked()), this, SLOT(etape23()) );
    // Mise à jour des informations pour les boutons et le menu déroulant
    pushButton_Rotation->setEnabled(false);
    pushButton_Control->setEnabled(false);
    _DisplayMouvement->setEnabled(true);
    pushButton_Control->setText(QString::fromUtf8("Enregistrer"));
    // Mise à jour du texte d'aide
    emit updateText(QString::fromUtf8(
        "Vous devez cliquer sur le centre des 3 diodes.\n"
        "Pour annuler un rond, il suffit de cliquer droit dessus.\n"
        "Attention : il est important de bien faire cette étape.\n"
        "Une fois cela effectué, vous pouvez enregistrer.\n"
    ));
}
//
/********************************************************************************************************/
//
void UIZor::etape23(){
    comboBox_Etape->setEnabled(true);
    // Connexion/Deconnexion des signaux et slots
    _DisplayMouvement->disconnect();
    pushButton_Control->disconnect();
    connect(pushButton_Control, SIGNAL(clicked()), this, SLOT(etapeChoix()) );
    pushButton_Control->setEnabled(true);
    pushButton_Control->setText("Retour");
    if(sauvegarde2()){
        emit updateText(QString::fromUtf8(
            "Félicitation, le calibrage du mouvement a été enregistré avec succès.\n"
            "Vous pouvez procéder au calibrage des couleurs ou quitter.\n"
        ));
    } else {
        emit updateText(QString::fromUtf8(
            "Le calibrage n'a pas pu être enregistré, consultez l'aide pour plus d'informations.\n"
        ));
    }
    _DisplayMouvement->setEnabled(false);
    _DisplayMouvement->effaceElements();
}
//
/********************************************************************************************************/
//
bool UIZor::sauvegarde1(){
    // Sauvegarde du seuil
    bool success=false;
    Individu individu= _Seuil->getIndividu();
    QFile file(QDir::currentPath()+"/calibrage/calibrageCouleurs.gc");
    if( file.open( QIODevice::WriteOnly | QIODevice::Text ) ){
        QTextStream output(&file);
        output << individu.rgb[0].x() << " " << individu.rgb[0].y() << " ";
        output << individu.rgb[1].x() << " " << individu.rgb[1].y() << " ";
        output << individu.rgb[2].x() << " " << individu.rgb[2].y() << endl;
        file.close();
        success=true;
    }
    return success;

}
//
/********************************************************************************************************/
//
bool UIZor::sauvegarde2(){
    // Sauvegarde des points pour le mouvement
    bool success=false;
    QFile file(QDir::currentPath()+"/calibrage/calibrageMouvement.gc");
    if( file.open( QIODevice::WriteOnly | QIODevice::Text ) ){
        QTextStream output(&file);
        output << _DisplayMouvement->_Element.at(0)->rectangle().center().x() << " " << _DisplayMouvement->_Element.at(0)->rectangle().center().y() << " ";
        output << _DisplayMouvement->_Element.at(1)->rectangle().center().x() << " " << _DisplayMouvement->_Element.at(1)->rectangle().center().y() << " ";
        output << _DisplayMouvement->_Element.at(2)->rectangle().center().x() << " " << _DisplayMouvement->_Element.at(2)->rectangle().center().y() << endl;
        file.close();
        success=true;
    }
    return success;

}
//
/********************************************************************************************************/
//




