/********************************************************************************************************/
/*
Copyright (C) 2008 Biger Kevin, Marguet Quentin, Muller Thomas, Vinatier FrÈdÈric (University of Science and Technology of Nantes)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

Contact: glovecontrol@free.fr
*/
/********************************************************************************************************/
//
#ifndef Seuil_H
#define Seuil_H
//
// Librairies QT
#include <QThread>
#include <QMutex>
#include <QTime>
#include <QRectF>
#include <QDir>
#include <QFileDialog>
#include <QPainter>
#include <QImage>
// Librairies OpenCV
#ifdef WIN32 // l'ordinateur est sous windows
#include <cv.h>
#include <highgui.h>
#else // Ordinateur sous autre chose que windows
#include <OpenCV/cv.h>
#include <OpenCV/highgui.h>
#endif  // endif
// Librairies Autres (stl...)
#include <time.h>
#include <cmath>
#include <iostream>
#include <limits>
#include "../Tools/OCVImage.h"
// Librairies Zor
#include "Tools.h"
#include "DisplayCouleurs.h"
// Librairies Blitz++
#include <blitz/array.h>
//
namespace Zor {
//
class Seuil : public QThread {
    //
	Q_OBJECT
	//
public:
    //
	Seuil( DisplayCouleurs * display, const Zor::Tools::TYPE_HEURISTIQUE& heuristique=Zor::Tools::HEU_0);
	~Seuil();
	//
    void run();
    //
    inline QImage& getImageSeuil() { return _ImageSeuil; }
    inline const Zor::Tools::Individu& getIndividu() { return _Individu; }
    inline void setHeuristique(const Zor::Tools::TYPE_HEURISTIQUE& heuristique) { _Heuristique = heuristique; }
    //
signals:
    //
    void calculTermine();
    void rafraichitProgression(int i);
    //
private:
    //
    void generePopulationAleatoire(blitz::Array<Zor::Tools::Individu,1>& tableauEntree, const QPoint& zoneEntree, const QPoint intervalleRGB[]); // Générer aléatoirement des Gènes
    void elite(blitz::Array<Zor::Tools::Individu,1>& tableauEntree, blitz::Array<Zor::Tools::Individu,1>& tableauSortie, const QPoint& zoneSortie);
    void mutation(blitz::Array<Zor::Tools::Individu,1>& tableauEntree, blitz::Array<Zor::Tools::Individu,1>& tableauSortie, const QPoint& zoneEntree, const QPoint& zoneSortie, const QPoint zoneAleatoire[], const QPoint intervalRGB[]);
    void croisement(blitz::Array<Zor::Tools::Individu,1>& tableauEntree, blitz::Array<Zor::Tools::Individu,1>& tableauSortie, const QPoint& zoneEntree, const QPoint& zoneSortie, const QPoint intervalRGB[]);
    void affichage(blitz::Array<Zor::Tools::Individu,1>& tableauEntree, const QPoint& zoneEntree);
    void genereImageSeuil(const Zor::Tools::Individu& seuil);
    //
    DisplayCouleurs * _Display;
    QImage * _QImage;
    QRect _Zone[3];
    QMutex mutex;
    QImage _ImageSeuil;
    Zor::Tools::Individu _Individu;
    Zor::Tools::TYPE_HEURISTIQUE _Heuristique;
    //
};  // Seuil
//
}   // ::Zor
//
#endif // Seuil_H
//
/********************************************************************************************************/
