/********************************************************************************************************/
/*
Copyright (C) 2008 Biger Kevin, Marguet Quentin, Muller Thomas, Vinatier Frédéric (University of Science and Technology of Nantes)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

Contact: glovecontrol@free.fr
*/
/********************************************************************************************************/
//
#include "Tools.h"
//
using namespace blitz;
//
namespace Zor {
namespace Tools {
//
/*************************************************************************/
//
Divers::~Divers() {
}
//
/*************************************************************************/
//
double Divers::calculRayonSelonResolution(const QRect& rect1, const QRect& rect2, const QRect& rect3, const QPoint& resolution){
    double d1, d2, d3;
    QPoint p1, p2, p3;
	QPointF coefficient;
	QDesktopWidget ecran;
	QPoint resolutionEcran = QPoint(ecran.screen()->width(),ecran.screen()->height());
    if((resolution.x()==0) || (resolution.y()==0)){
        coefficient = QPointF(1.0 ,1.0);
    } else {
        coefficient = QPointF((float)resolutionEcran.x() / resolution.x() , (float)resolutionEcran.y() / resolution.y());
    }

    p1.rx() = static_cast<int>(p1.x() * coefficient.x()); p1.ry() = static_cast<int>(p1.y() * coefficient.y());;
    p2.rx() = static_cast<int>(p2.x() * coefficient.x()); p2.ry() = static_cast<int>(p2.y() * coefficient.y());
    p3.rx() = static_cast<int>(p3.x() * coefficient.x()); p3.ry() = static_cast<int>(p3.y() * coefficient.y());

    std::cout << "coef : " <<  coefficient.x() << " " << coefficient.y() << std::endl;
	p1 = rect1.center();
	p2 = rect2.center();
	p3 = rect3.center();

	std::cout << "centre des diodes 1 : " << p1.x() << " " << p1.y() << std::endl;
	std::cout << "centre des diodes 2 : " << p2.x() << " " << p2.y() << std::endl;
	std::cout << "centre des diodes 3 : " << p3.x() << " " << p3.y() << std::endl;
	QPoint center((p1.x()+p2.x()+p3.x())/3,(p1.y()+p2.y()+p3.y())/3);


    std::cout << "Centre des diodes : (" << center.x() << ", " << center.y() << ")" << std::endl;
	d1 = pow( static_cast<double>(p1.x() - center.x()) ,2) + pow( static_cast<double>(p1.y() - center.y()) ,2) ;
	d2 = pow( static_cast<double>(p2.x() - center.x()) ,2) + pow( static_cast<double>(p2.y() - center.y()) ,2) ;
	d3 = pow( static_cast<double>(p3.x() - center.x()) ,2) + pow( static_cast<double>(p3.y() - center.y()) ,2) ;
	std::cout << "Distance 1 : " << sqrt(d1) << std::endl;
	std::cout << "Distance 2 : " << sqrt(d2) << std::endl;
	std::cout << "Distance 3 : " << sqrt(d3) << std::endl;
    std::cout << "Rayon : " << sqrt(std::max(d3,std::max(d1,d2))) << std::endl;
	return sqrt(std::max(d3,std::max(d1,d2)));
}
//
/*************************************************************************/
// Retourne une valeur aléatoire (min <= aleatoire < max)
int Divers::aleatoire(const int& min, const int& max){
    int resultat = (int) (min+((float)rand()/RAND_MAX*(max-min)));
    if(resultat<min) {
        return min;
    } else if(resultat>=max) {
        return max-1;
    }
    return resultat;
}
//
/*************************************************************************/
//
bool Divers::dansSeuil(const QColor& couleur, const Individu& filtre){
    return (
            (filtre.rgb[0].x() <= couleur.red()) && (couleur.red() <= filtre.rgb[0].y())          // Couleur dans le filtre rouge ?
        &&  (filtre.rgb[1].x() <= couleur.green()) && (couleur.green() <= filtre.rgb[1].y())      // Couleur dans le filtre vert ?
        &&  (filtre.rgb[2].x() <= couleur.blue()) && (couleur.blue() <= filtre.rgb[2].y())        // Couleur dans le filtre bleu ?
        );
}
//
/*************************************************************************/
// Tri rapide dans une zone (min <= zone < max)
void Divers::triRapide(Array<Individu,1>& tableauEntree, const QPoint& zoneEntree) {
    if(zoneEntree.x()<zoneEntree.y()) {
        int partition = Divers::partitionner(tableauEntree, zoneEntree);
        triRapide(tableauEntree, QPoint(zoneEntree.x(),partition) );
        triRapide(tableauEntree, QPoint(partition+1,zoneEntree.y()) );
    }
}
//
/********************************************************************************************************/
//
int Divers::partitionner(Array<Individu,1>& tableauEntree, const QPoint& zoneEntree) {
    double pivot = tableauEntree(zoneEntree.x()).valeur;
    int i = zoneEntree.x()-1, j = zoneEntree.y();
    Array<Individu,1> tableauSauvegarde(1);
    while(1) {
        do { j--; } while(tableauEntree(j).valeur < pivot);
        do { i++; } while(tableauEntree(i).valeur > pivot);
        if(i<j) {
            tableauSauvegarde(0) = tableauEntree(i);
            tableauEntree(i) = tableauEntree(j);
            tableauEntree(j) = tableauSauvegarde(0);
        } else {
            return j;
        }
    }
    return j;
}
//
/********************************************************************************************************/
//
QImage * Divers::convertionIplImageQImage(IplImage * image, const TYPE_CONVERSION& type ) {

    // Creation de l'image de sortie
    QImage * imageSortie = new QImage( image->width, image->height, QImage::Format_RGB32 );
    //int t = QSettings::Windows; // 0 : unix, 1 : windows, 2 : mac
    switch(type){
        case INVERSION:
            // Boucle de parcours pour mettre à jour les pixels de la QImage
            for(int y = 0; y < image->height; y++) {
                for(int x = 0; x < image->width; x++ ) {
                    unsigned char * pixel = &((uchar*)(image->imageData + image->widthStep*y))[x*3];
                    QColor couleur(pixel[2],pixel[1],pixel[0],0);
                    imageSortie->setPixel(x,y,couleur.rgba());
                }
            }
        break;
        default:
            // Boucle de parcours pour mettre à jour les pixels de la QImage
            int largeur = image->width-1;
            int longueur = image->height-1;
            for(int y = 0; y < image->height; y++) {
                for(int x = 0; x < image->width; x++ ) {
                    unsigned char * pixel = &((uchar*)(image->imageData + image->widthStep*y))[x*3];
                    QColor couleur(pixel[2],pixel[1],pixel[0],0);
                    imageSortie->setPixel(largeur-x,longueur-y,couleur.rgba());
                }
            }
        break;
    }


    // Retour de l'image sous format QImage
    return imageSortie;
}
//
/********************************************************************************************************/
//
ZorRect::ZorRect(const QRect& rectangle, const bool& existe, const bool& traite, const bool& selectionne){
    _Rectangle=rectangle;
    _Existe=existe;
    _Traite=traite;
}
//
/*************************************************************************/
//
ZorRect::~ZorRect(){}
//
/*************************************************************************/
//
QRect& ZorRect::rectangle(){
    return _Rectangle;
}
//
/*************************************************************************/
//
bool& ZorRect::existe(){
    return _Existe;
}
//
/*************************************************************************/
//
bool& ZorRect::traite(){
    return _Traite;
}
//
/*************************************************************************/
//
bool& ZorRect::selectionne(){
    return _Selectionne;
}
//
/*************************************************************************/
//
}   // Tools
}   // Zor
//
/*************************************************************************/
//

